--Information about the physical memory for the server

SELECT 
        total_physical_memory_kb/1024 AS 'Total Physical Memory MB'
	  , available_physical_memory_kb/1024 AS 'Available Physical Memory MB'
	  , total_page_file_kb/1024 AS 'Total Page File MB'
	  , available_page_file_kb/1024 AS 'Available Page File MB'
	  , 100 - (100 * CAST(available_physical_memory_kb AS DECIMAL(18,3))/CAST(total_physical_memory_kb AS DECIMAL(18,3))) 
				AS 'Percentage Used'
	  , system_memory_state_desc
FROM  sys.dm_os_sys_memory;

--============================================================================
--============================================================================

SELECT Name
		, value_in_use
FROM sys.configurations
WHERE Name IN ('query wait (s)', 'min memory per query (KB)')
UNION
SELECT    [name]
		, value_in_use 
FROM sys.configurations 
WHERE [name] LIKE 'max server memory%'
UNION
SELECT	  [name]
		, value_in_use 
FROM sys.configurations 
WHERE [name] LIKE 'min server memory%'


--============================================================================
--============================================================================

--From https://www.sqlshack.com/insight-into-the-sql-server-buffer-cache/
--Number of database pages by database

SELECT
    databases.name AS 'DatabaseName',
	Count(*) 'NumberOfPages',
    COUNT(*) * 8 / 1024 AS 'MBUsed',
	'www.sqlshack.com/insight-into-the-sql-server-buffer-cache/' AS 'Source'
FROM sys.dm_os_buffer_descriptors
INNER JOIN sys.databases
ON databases.database_id = dm_os_buffer_descriptors.database_id
GROUP BY databases.name
ORDER BY MBUsed DESC;



--============================================================================
--=========================================================================

--https://blog.sqlauthority.com/2017/10/20/sql-server-turn-optimize-ad-hoc-workloads/
--Pinal Dave
--Percentage of queries that are Ad-Hoc queries

SELECT 'Percent of Cache - Ad Hoc Queries' AS 'Status',
		AdHoc_Plan_MB, Total_Cache_MB,
		AdHoc_Plan_MB*100.0 / Total_Cache_MB AS 'AdHoc %',
		'Pinal Dave - blog.sqlauthority.com/2017/10/20/sql-server-turn-optimize-ad-hoc-workloads/' AS 'Source'
FROM (
SELECT SUM(CASE
			WHEN objtype = 'adhoc'
			THEN CONVERT(BIGINT,size_in_bytes)
			ELSE 0 END) / 1048576.0 AdHoc_Plan_MB,
		SUM(CONVERT(BIGINT,size_in_bytes)) / 1048576.0 Total_Cache_MB
FROM sys.dm_exec_cached_plans) T


--============================================================================
--============================================================================

--Capture memory related performance counters 

DECLARE @totalMem BIGINT

SELECT @totalMem = cntr_value
FROM sys.dm_os_performance_counters
WHERE [counter_name] = 'Target Server Memory (KB)'

SELECT    [object_name]
		, counter_name
		, cntr_value
		, CASE  WHEN counter_name = 'Page Life Expectancy'		THEN 0
				WHEN counter_name = 'SQL Cache Memory (KB)'		THEN CONVERT(DECIMAL(32,2),cntr_value) / @totalMem
				WHEN counter_name = 'Stolen Server Memory (KB)' THEN CONVERT(DECIMAL(32,2),cntr_value) / @totalMem
				WHEN counter_name = 'Connection Memory (KB)'	THEN CONVERT(DECIMAL(32,2),cntr_value) / @totalMem
				WHEN counter_name = 'Free Memory (KB)'			THEN CONVERT(DECIMAL(32,2),cntr_value) / @totalMem
				WHEN counter_name = 'Optimizer Memory (KB)'		THEN CONVERT(DECIMAL(32,2),cntr_value) / @totalMem
				WHEN counter_name = 'Lock Memory (KB)'			THEN CONVERT(DECIMAL(32,2),cntr_value) / @totalMem
				WHEN counter_name = 'Total Server Memory (KB)'	THEN CONVERT(DECIMAL(32,2),cntr_value) / @totalMem
				WHEN counter_name = 'Target Server Memory (KB)'	THEN CONVERT(DECIMAL(32,2),cntr_value) / @totalMem
				WHEN counter_name = 'Database pages'			THEN (CONVERT(DECIMAL(32,2),cntr_value)*8) / @totalMem
				WHEN counter_name = 'Memory Grants Outstanding'	THEN 0
				WHEN counter_name = 'Memory Grants Pending'		THEN 0
				WHEN counter_name = 'Cursor memory usage'		THEN CONVERT(DECIMAL(32,2),cntr_value) 
				WHEN counter_name = 'Checkpoint pages/sec'		THEN 0
				WHEN counter_name = 'Page reads/sec'			THEN 0
				WHEN counter_name = 'Page writes/sec'			THEN 0 
				WHEN counter_name = 'Batch Requests/sec'		THEN 0
				WHEN counter_name = 'SQL Compilations/sec'		THEN 0
				WHEN counter_name = 'SQL re-Compilations/sec'	THEN 0
				WHEN counter_name = 'Free list stalls/sec'		THEN 0
				WHEN counter_name = 'Lazy writes/sec'			THEN 0   
		  ELSE
				0
		  END AS 'PercentOfTotalMemoryUsed(KB)'
		, CASE  WHEN counter_name = 'Page Life Expectancy'		THEN 0
				WHEN counter_name = 'SQL Cache Memory (KB)'		THEN CONVERT(DECIMAL(32,2),cntr_value) 
				WHEN counter_name = 'Stolen Server Memory (KB)' THEN CONVERT(DECIMAL(32,2),cntr_value) 
				WHEN counter_name = 'Connection Memory (KB)'	THEN CONVERT(DECIMAL(32,2),cntr_value) 
				WHEN counter_name = 'Free Memory (KB)'			THEN CONVERT(DECIMAL(32,2),cntr_value) 
				WHEN counter_name = 'Optimizer Memory (KB)'		THEN CONVERT(DECIMAL(32,2),cntr_value) 
				WHEN counter_name = 'Lock Memory (KB)'			THEN CONVERT(DECIMAL(32,2),cntr_value) 
				WHEN counter_name = 'Total Server Memory (KB)'	THEN 0 
				WHEN counter_name = 'Target Server Memory (KB)'	THEN 0 
				WHEN counter_name = 'Database pages'			THEN CONVERT(DECIMAL(32,2),cntr_value) * 8
				WHEN counter_name = 'Memory Grants Outstanding'	THEN 0
				WHEN counter_name = 'Memory Grants Pending'		THEN 0
				WHEN counter_name = 'Cursor memory usage'		THEN CONVERT(DECIMAL(32,2),cntr_value) 
				WHEN counter_name = 'Checkpoint pages/sec'		THEN 0 
				WHEN counter_name = 'Page reads/sec'			THEN 0 
				WHEN counter_name = 'Page writes/sec'			THEN 0
				WHEN counter_name = 'Batch Requests/sec'		THEN 0 
				WHEN counter_name = 'SQL Compilations/sec'		THEN 0 
				WHEN counter_name = 'SQL re-Compilations/sec'	THEN 0
				WHEN counter_name = 'Free list stalls/sec'		THEN 0
				WHEN counter_name = 'Lazy writes/sec'			THEN 0    
		  ELSE
				0
		  END AS 'SpaceConsumedInMemory'
FROM sys.dm_os_performance_counters
WHERE [counter_name] IN (	  'Page Life Expectancy'
							, 'Stolen Server Memory (KB)'
							, 'Total Server Memory (KB)'
							, 'Target Server Memory (KB)'
							, 'SQL Cache Memory (KB)'
							, 'Optimizer Memory (KB)'
							, 'Memory Grants Outstanding'
							, 'Memory Grants Pending'
							, 'Free Memory (KB)'
							, 'Connection Memory (KB)'
							, 'Lock Memory (KB)'
							, 'Cursor memory usage'
							, 'Database pages'
							, 'Checkpoint pages/sec'
							, 'Page reads/sec'
							, 'Page writes/sec'
							, 'Batch Requests/sec'
							, 'SQL Compilations/sec'
							, 'SQL re-Compilations/sec'
							, 'Free list stalls/sec'
							, 'Lazy writes/sec'
							, 'Granted Workspace Memory (KB)'
							, 'Maximum Workspace Memory (KB)' 
							, 'Average Wait Time (ms)'
							, 'Lock Waits/sec'
							, 'Active cursors'
							, 'Cursor Requests/sec'
							, 'Memory broker clerk size'
							, 'Pressure evictions/sec')
							AND instance_name IN ('','_total')



	
	DECLARE @iCount AS SMALLINT
	CREATE TABLE #CounterValues
				(  Counter_name      VARCHAR(50)
				 , Currentvalue		 INT
				 , DateTimeOfCapture DATETIME)
	SET @iCount = 1

	WHILE @iCount < 4
		BEGIN

			INSERT INTO #CounterValues
			SELECT    Counter_Name
					, cntr_value 
					, GETDATE()
			FROM sys.dm_os_performance_counters
			WHERE counter_name IN
							(   'Free list stalls/sec'
							  , 'Lazy writes/sec'
							  , 'Batch Requests/sec'
							  , 'SQL Compilations/sec')

			WAITFOR DELAY '00:00:05'

			SET @icount = @icount + 1
		END



	


DECLARE @cols   AS NVARCHAR(MAX)
DECLARE @query  AS NVARCHAR(MAX);


SET @cols = STUFF((SELECT DISTINCT ',' + QUOTENAME(
					CONVERT(VARCHAR(40),LEFT(CONVERT(CHAR(40),DateTimeOfCapture, 109) , 20))) AS DateTimeOfCapture
					FROM #CounterValues
						ORDER BY  DateTimeOfCapture ASC
            FOR XML PATH(''), TYPE
            ).value('.', 'NVARCHAR(MAX)') 
			,1,1,'')


SET @query = 'SELECT  Counter_Name,' + @cols + ' from 
            (
				SELECT   Counter_Name, currentvalue, LEFT(CONVERT(CHAR(40),DateTimeOfCapture, 109) , 20) AS TimeOfCapture
				FROM #CounterValues
			) x
            PIVOT 
            (
                SUM(Currentvalue)
				FOR TimeOfCapture IN  (' + @cols + ')
            ) p '
 
 
EXECUTE(@query)

--Returns detail rows for confirmation

SELECT   * , left(convert(char(40),DateTimeOfCapture, 109) , 20) AS 'FormatedDate'
FROM #CounterValues
ORDER BY Counter_name, DateTimeOfCapture

DROP TABLE #CounterValues
